"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildTokenId = exports.isNativeCosmWasmDenom = exports.isNativeDenomProvenance = exports.isNativeDenomDymension = exports.isNativeDenomSeda = exports.isNativeDenomStargaze = exports.isNativeDenomCelestia = exports.isNativeDenomNeutron = exports.isNativeDenomKujira = exports.isNativeDenomEvmos = exports.isNativeDenomCosmosHub = exports.isNativeDenomOsmosis = exports.isNativeDenomWormchain = exports.isNativeDenomSei = exports.isNativeDenomXpla = exports.isNativeDenomInjective = void 0;
const js_sha3_1 = require("js-sha3");
const utils_1 = require("../utils");
const isNativeDenomInjective = (denom) => denom === "inj";
exports.isNativeDenomInjective = isNativeDenomInjective;
const isNativeDenomXpla = (denom) => denom === "axpla";
exports.isNativeDenomXpla = isNativeDenomXpla;
const isNativeDenomSei = (denom) => denom === "usei";
exports.isNativeDenomSei = isNativeDenomSei;
const isNativeDenomWormchain = (denom) => denom === "uworm";
exports.isNativeDenomWormchain = isNativeDenomWormchain;
const isNativeDenomOsmosis = (denom) => denom === "uosmo";
exports.isNativeDenomOsmosis = isNativeDenomOsmosis;
const isNativeDenomCosmosHub = (denom) => denom === "uatom";
exports.isNativeDenomCosmosHub = isNativeDenomCosmosHub;
const isNativeDenomEvmos = (denom) => denom === "aevmos" || denom === "atevmos";
exports.isNativeDenomEvmos = isNativeDenomEvmos;
const isNativeDenomKujira = (denom) => denom === "ukuji";
exports.isNativeDenomKujira = isNativeDenomKujira;
const isNativeDenomNeutron = (denom) => denom === "untrn";
exports.isNativeDenomNeutron = isNativeDenomNeutron;
const isNativeDenomCelestia = (denom) => denom === "utia";
exports.isNativeDenomCelestia = isNativeDenomCelestia;
const isNativeDenomStargaze = (denom) => denom === "ustars";
exports.isNativeDenomStargaze = isNativeDenomStargaze;
const isNativeDenomSeda = (denom) => denom === "aseda";
exports.isNativeDenomSeda = isNativeDenomSeda;
const isNativeDenomDymension = (denom) => denom === "adym";
exports.isNativeDenomDymension = isNativeDenomDymension;
const isNativeDenomProvenance = (denom) => denom === "nhash";
exports.isNativeDenomProvenance = isNativeDenomProvenance;
function isNativeCosmWasmDenom(chainId, address) {
    return ((chainId === utils_1.CHAIN_ID_INJECTIVE && (0, exports.isNativeDenomInjective)(address)) ||
        (chainId === utils_1.CHAIN_ID_XPLA && (0, exports.isNativeDenomXpla)(address)) ||
        (chainId === utils_1.CHAIN_ID_SEI && (0, exports.isNativeDenomSei)(address)) ||
        (chainId === utils_1.CHAIN_ID_WORMCHAIN && (0, exports.isNativeDenomWormchain)(address)) ||
        (chainId === utils_1.CHAIN_ID_OSMOSIS && (0, exports.isNativeDenomOsmosis)(address)) ||
        (chainId === utils_1.CHAIN_ID_COSMOSHUB && (0, exports.isNativeDenomCosmosHub)(address)) ||
        (chainId === utils_1.CHAIN_ID_EVMOS && (0, exports.isNativeDenomEvmos)(address)) ||
        (chainId === utils_1.CHAIN_ID_KUJIRA && (0, exports.isNativeDenomKujira)(address)) ||
        (chainId === utils_1.CHAIN_ID_NEUTRON && (0, exports.isNativeDenomNeutron)(address)) ||
        (chainId === utils_1.CHAIN_ID_CELESTIA && (0, exports.isNativeDenomCelestia)(address)) ||
        (chainId === utils_1.CHAIN_ID_STARGAZE && (0, exports.isNativeDenomStargaze)(address)) ||
        (chainId === utils_1.CHAIN_ID_SEDA && (0, exports.isNativeDenomSeda)(address)) ||
        (chainId === utils_1.CHAIN_ID_DYMENSION && (0, exports.isNativeDenomDymension)(address)) ||
        (chainId === utils_1.CHAIN_ID_PROVENANCE && (0, exports.isNativeDenomProvenance)(address)));
}
exports.isNativeCosmWasmDenom = isNativeCosmWasmDenom;
function buildTokenId(chain, address) {
    const chainId = (0, utils_1.coalesceCosmWasmChainId)(chain);
    return ((isNativeCosmWasmDenom(chainId, address) ? "01" : "00") +
        (0, js_sha3_1.keccak256)(Buffer.from(address, "utf-8")).substring(4));
}
exports.buildTokenId = buildTokenId;
