import { keccak256 } from "js-sha3";
import { CHAIN_ID_COSMOSHUB, CHAIN_ID_EVMOS, CHAIN_ID_WORMCHAIN, CHAIN_ID_INJECTIVE, CHAIN_ID_SEI, CHAIN_ID_XPLA, coalesceCosmWasmChainId, CHAIN_ID_KUJIRA, CHAIN_ID_NEUTRON, CHAIN_ID_OSMOSIS, CHAIN_ID_CELESTIA, CHAIN_ID_STARGAZE, CHAIN_ID_SEDA, CHAIN_ID_DYMENSION, CHAIN_ID_PROVENANCE, } from "../utils";
export const isNativeDenomInjective = (denom) => denom === "inj";
export const isNativeDenomXpla = (denom) => denom === "axpla";
export const isNativeDenomSei = (denom) => denom === "usei";
export const isNativeDenomWormchain = (denom) => denom === "uworm";
export const isNativeDenomOsmosis = (denom) => denom === "uosmo";
export const isNativeDenomCosmosHub = (denom) => denom === "uatom";
export const isNativeDenomEvmos = (denom) => denom === "aevmos" || denom === "atevmos";
export const isNativeDenomKujira = (denom) => denom === "ukuji";
export const isNativeDenomNeutron = (denom) => denom === "untrn";
export const isNativeDenomCelestia = (denom) => denom === "utia";
export const isNativeDenomStargaze = (denom) => denom === "ustars";
export const isNativeDenomSeda = (denom) => denom === "aseda";
export const isNativeDenomDymension = (denom) => denom === "adym";
export const isNativeDenomProvenance = (denom) => denom === "nhash";
export function isNativeCosmWasmDenom(chainId, address) {
    return ((chainId === CHAIN_ID_INJECTIVE && isNativeDenomInjective(address)) ||
        (chainId === CHAIN_ID_XPLA && isNativeDenomXpla(address)) ||
        (chainId === CHAIN_ID_SEI && isNativeDenomSei(address)) ||
        (chainId === CHAIN_ID_WORMCHAIN && isNativeDenomWormchain(address)) ||
        (chainId === CHAIN_ID_OSMOSIS && isNativeDenomOsmosis(address)) ||
        (chainId === CHAIN_ID_COSMOSHUB && isNativeDenomCosmosHub(address)) ||
        (chainId === CHAIN_ID_EVMOS && isNativeDenomEvmos(address)) ||
        (chainId === CHAIN_ID_KUJIRA && isNativeDenomKujira(address)) ||
        (chainId === CHAIN_ID_NEUTRON && isNativeDenomNeutron(address)) ||
        (chainId === CHAIN_ID_CELESTIA && isNativeDenomCelestia(address)) ||
        (chainId === CHAIN_ID_STARGAZE && isNativeDenomStargaze(address)) ||
        (chainId === CHAIN_ID_SEDA && isNativeDenomSeda(address)) ||
        (chainId === CHAIN_ID_DYMENSION && isNativeDenomDymension(address)) ||
        (chainId === CHAIN_ID_PROVENANCE && isNativeDenomProvenance(address)));
}
export function buildTokenId(chain, address) {
    const chainId = coalesceCosmWasmChainId(chain);
    return ((isNativeCosmWasmDenom(chainId, address) ? "01" : "00") +
        keccak256(Buffer.from(address, "utf-8")).substring(4));
}
